@@ -1,3 +1,5 @@
+from enum import Enum
+
 import six
 
 from dagster import check
@@ -289,6 +291,9 @@ def validate_enum_config(context, config_value):
     check.invariant(context.config_type.kind == ConfigTypeKind.ENUM)
     check.not_none_param(config_value, 'config_value')
 
+    if isinstance(config_value, Enum):
+        config_value = config_value.name
+
     if not isinstance(config_value, six.string_types):
         return EvaluateValueResult.for_error(create_enum_type_mismatch_error(context, config_value))
 
