@@ -2,7 +2,14 @@ import os
 import sys
 
 from dagster import EventMetadataEntry, check
-from dagster.core.definitions import ExpectationResult, Materialization, Output, TypeCheck
+from dagster.core.definitions import (
+    ExpectationResult,
+    Failure,
+    Materialization,
+    Output,
+    RetryRequested,
+    TypeCheck,
+)
 from dagster.core.errors import (
     DagsterError,
     DagsterExecutionStepExecutionError,
@@ -27,14 +34,15 @@ from dagster.core.execution.plan.objects import (
     StepInputData,
     StepOutputData,
     StepOutputHandle,
+    StepRetryData,
     StepSuccessData,
     TypeCheckData,
     UserFailureData,
 )
 from dagster.core.execution.plan.plan import ExecutionPlan
 from dagster.core.storage.object_store import ObjectStoreOperation
-from dagster.core.types.dagster_type import DagsterType
-from dagster.utils.error import serializable_error_info_from_exc_info
+from dagster.core.types.dagster_type import DagsterType, DagsterTypeKind
+from dagster.utils.error import SerializableErrorInfo, serializable_error_info_from_exc_info
 from dagster.utils.timing import format_duration, time_execution_scope
 
 from .engine_base import Engine
@@ -46,12 +54,18 @@ class InProcessEngine(Engine):  # pylint: disable=no-init
         check.inst_param(pipeline_context, 'pipeline_context', SystemPipelineExecutionContext)
         check.inst_param(execution_plan, 'execution_plan', ExecutionPlan)
 
+        step_keys_to_execute = execution_plan.step_keys_to_execute
+        for_step_key = (
+            step_keys_to_execute[0]
+            if step_keys_to_execute and len(step_keys_to_execute) == 1
+            else None
+        )
+
         yield DagsterEvent.engine_event(
             pipeline_context,
             'Executing steps in process (pid: {pid})'.format(pid=os.getpid()),
-            event_specific_data=EngineEventData.in_process(
-                os.getpid(), execution_plan.step_keys_to_execute
-            ),
+            event_specific_data=EngineEventData.in_process(os.getpid(), step_keys_to_execute),
+            step_key=for_step_key,
         )
 
         with time_execution_scope() as timer_result:
@@ -71,14 +85,12 @@ class InProcessEngine(Engine):  # pylint: disable=no-init
             # It would be good to implement a reference tracking algorithm here to
             # garbage collect results that are no longer needed by any steps
             # https://github.com/dagster-io/dagster/issues/811
-            active_execution = execution_plan.start()
+            active_execution = execution_plan.start(
+                retries=pipeline_context.executor_config.retries
+            )
             while not active_execution.is_complete:
+                step = active_execution.get_next_step()
 
-                steps = active_execution.get_steps_to_execute(limit=1)
-                check.invariant(
-                    len(steps) == 1, 'Invariant Violation: expected step to be available to execute'
-                )
-                step = steps[0]
                 step_context = pipeline_context.for_step(step)
                 check.invariant(
                     all(
@@ -125,9 +137,8 @@ class InProcessEngine(Engine):  # pylint: disable=no-init
             'Finished steps in process (pid: {pid}) in {duration_ms}'.format(
                 pid=os.getpid(), duration_ms=format_duration(timer_result.millis)
             ),
-            event_specific_data=EngineEventData.in_process(
-                os.getpid(), execution_plan.step_keys_to_execute
-            ),
+            event_specific_data=EngineEventData.in_process(os.getpid(), step_keys_to_execute),
+            step_key=for_step_key,
         )
 
 
@@ -155,7 +166,7 @@ def _input_values_from_intermediates_manager(step_context):
 
     input_values = {}
     for step_input in step.step_inputs:
-        if step_input.runtime_type.is_nothing:
+        if step_input.runtime_type.kind == DagsterTypeKind.NOTHING:
             continue
 
         if step_input.is_from_multiple_outputs:
@@ -225,23 +236,33 @@ def dagster_event_sequence_for_step(step_context):
 
     Thie function also processes errors. It handles a few error cases:
 
-        (1) The user-space code has raised an Exception. It has been
-        wrapped in an exception derived from DagsterUserCodeException. In that
-        case the original user exc_info is stashed on the exception
-        as the original_exc_info property. Examples of this are computations
-        with the compute_fn, and type checks. If the user has raised an
-        intentional error via throwing Failure, they can also optionally
-        pass along explicit metadata attached to that Failure.
+        (1) User code requests to be retried:
+            A RetryRequested has been raised. We will either put the step in to
+            up_for_retry state or a failure state depending on the number of previous attempts
+            and the max_retries on the recieved RetryRequested.
 
-        (2) The framework raised a DagsterError that indicates a usage error
-        or some other error not communicated by a user-thrown exception. For example,
-        if the user yields an object out of a compute function that is not a
-        proper event (not an Output, ExpectationResult, etc).
+        (2) User code fails successfully:
+            The user-space code has raised a Failure which may have
+            explicit metadata attached.
+
+        (3) User code fails unexpectedly:
+            The user-space code has raised an Exception. It has been
+            wrapped in an exception derived from DagsterUserCodeException. In that
+            case the original user exc_info is stashed on the exception
+            as the original_exc_info property.
+
+        (4) User error:
+            The framework raised a DagsterError that indicates a usage error
+            or some other error not communicated by a user-thrown exception. For example,
+            if the user yields an object out of a compute function that is not a
+            proper event (not an Output, ExpectationResult, etc).
+
+        (5) Framework failure or interrupt:
+            An unexpected error occured. This is a framework error. Either there
+            has been an internal error in the framework OR we have forgtten to put a
+            user code error boundary around invoked user-space code. These terminate
+            the computation immediately (by re-raising).
 
-        (3) An unexpected error occured. This is a framework error. Either there
-        has been an internal error in the framewore OR we have forgtten to put a
-        user code error boundary around invoked user-space code. These terminate
-        the computation immediately (by re-raising).
 
     The "raised_dagster_errors" context manager can be used to force these errors to be
     reraised and surfaced to the user. This is mostly to get sensible errors in test and
@@ -259,30 +280,74 @@ def dagster_event_sequence_for_step(step_context):
             yield step_event
 
     # case (1) in top comment
-    except DagsterUserCodeExecutionError as dagster_user_error:  # case (1) above
+    except RetryRequested as retry_request:
+        retries = step_context.executor_config.retries
+        retry_err_info = serializable_error_info_from_exc_info(sys.exc_info())
+
+        if retries.disabled:
+            fail_err = SerializableErrorInfo(
+                message='RetryRequested but retries are disabled',
+                stack=retry_err_info.stack,
+                cls_name=retry_err_info.cls_name,
+                cause=retry_err_info.cause,
+            )
+            yield DagsterEvent.step_failure_event(
+                step_context=step_context,
+                step_failure_data=StepFailureData(error=fail_err, user_failure_data=None),
+            )
+        else:  # retries.enabled or retries.deferred
+            prev_attempts = retries.get_attempt_count(step_context.step.key)
+            if prev_attempts >= retry_request.max_retries:
+                fail_err = SerializableErrorInfo(
+                    message='Exceeded max_retries of {}'.format(retry_request.max_retries),
+                    stack=retry_err_info.stack,
+                    cls_name=retry_err_info.cls_name,
+                    cause=retry_err_info.cause,
+                )
+                yield DagsterEvent.step_failure_event(
+                    step_context=step_context,
+                    step_failure_data=StepFailureData(error=fail_err, user_failure_data=None),
+                )
+            else:
+                attempt_num = prev_attempts + 1
+                yield DagsterEvent.step_retry_event(
+                    step_context,
+                    StepRetryData(
+                        error=retry_err_info, seconds_to_wait=retry_request.seconds_to_wait,
+                    ),
+                )
+
+    # case (2) in top comment
+    except Failure as failure:
         yield _step_failure_event_from_exc_info(
             step_context,
-            dagster_user_error.original_exc_info,
+            sys.exc_info(),
             UserFailureData(
                 label='intentional-failure',
-                description=dagster_user_error.user_specified_failure.description,
-                metadata_entries=dagster_user_error.user_specified_failure.metadata_entries,
-            )
-            if dagster_user_error.is_user_specified_failure
-            else None,
+                description=failure.description,
+                metadata_entries=failure.metadata_entries,
+            ),
+        )
+        if step_context.raise_on_error:
+            raise failure
+
+    # case (3) in top comment
+    except DagsterUserCodeExecutionError as dagster_user_error:
+        yield _step_failure_event_from_exc_info(
+            step_context, dagster_user_error.original_exc_info,
         )
 
         if step_context.raise_on_error:
             raise dagster_user_error.user_exception
 
-    # case (2) in top comment
+    # case (4) in top comment
     except DagsterError as dagster_error:
         yield _step_failure_event_from_exc_info(step_context, sys.exc_info())
 
         if step_context.raise_on_error:
             raise dagster_error
 
-    # case (3) in top comment
+    # case (5) in top comment
     except (Exception, KeyboardInterrupt) as unexpected_exception:  # pylint: disable=broad-except
         yield _step_failure_event_from_exc_info(step_context, sys.exc_info())
 
@@ -332,7 +397,7 @@ def _step_output_error_checked_user_event_sequence(step_context, user_event_sequ
 
     for step_output_def in step.step_outputs:
         if not step_output_def.name in seen_outputs and not step_output_def.optional:
-            if step_output_def.runtime_type.is_nothing:
+            if step_output_def.runtime_type.kind == DagsterTypeKind.NOTHING:
                 step_context.log.info(
                     'Emitting implicit Nothing for output "{output}" on solid {solid}'.format(
                         output=step_output_def.name, solid={str(step.solid_handle)}
@@ -500,8 +565,11 @@ def _core_dagster_event_sequence_for_step(step_context):
     of the step.
     '''
     check.inst_param(step_context, 'step_context', SystemStepExecutionContext)
-
-    yield DagsterEvent.step_start_event(step_context)
+    attempts = step_context.executor_config.retries.get_attempt_count(step_context.step.key)
+    if attempts > 0:
+        yield DagsterEvent.step_restarted_event(step_context, attempts)
+    else:
+        yield DagsterEvent.step_start_event(step_context)
 
     inputs = {}
     for input_name, input_value in _input_values_from_intermediates_manager(step_context).items():
@@ -644,6 +712,7 @@ def _user_event_sequence_for_step_compute_fn(step_context, evaluated_inputs):
 
     with user_code_error_boundary(
         DagsterExecutionStepExecutionError,
+        control_flow_exceptions=[Failure, RetryRequested],
         msg_fn=lambda: '''Error occured during the execution of step:
         step key: "{key}"
         solid invocation: "{solid}"
