@@ -1,4 +1,5 @@
 '''Structured representations of system events.'''
+import logging
 from collections import namedtuple
 from enum import Enum
 
@@ -17,7 +18,7 @@ from dagster.core.execution.context.system import (
 )
 from dagster.core.execution.plan.objects import StepOutputData
 from dagster.core.log_manager import DagsterLogManager
-from dagster.core.serdes import whitelist_for_serdes
+from dagster.core.serdes import register_serdes_tuple_fallbacks, whitelist_for_serdes
 from dagster.utils.error import SerializableErrorInfo
 from dagster.utils.timing import format_duration
 
@@ -31,6 +32,10 @@ class DagsterEventType(Enum):
     STEP_START = 'STEP_START'
     STEP_SUCCESS = 'STEP_SUCCESS'
     STEP_SKIPPED = 'STEP_SKIPPED'
+
+    STEP_UP_FOR_RETRY = 'STEP_UP_FOR_RETRY'  # "failed" but want to retry
+    STEP_RESTARTED = 'STEP_RESTARTED'
+
     STEP_MATERIALIZATION = 'STEP_MATERIALIZATION'
     STEP_EXPECTATION_RESULT = 'STEP_EXPECTATION_RESULT'
 
@@ -40,10 +45,6 @@ class DagsterEventType(Enum):
     PIPELINE_SUCCESS = 'PIPELINE_SUCCESS'
     PIPELINE_FAILURE = 'PIPELINE_FAILURE'
 
-    PIPELINE_PROCESS_START = 'PIPELINE_PROCESS_START'
-    PIPELINE_PROCESS_STARTED = 'PIPELINE_PROCESS_STARTED'
-    PIPELINE_PROCESS_EXITED = 'PIPELINE_PROCESS_EXITED'
-
     OBJECT_STORE_OPERATION = 'OBJECT_STORE_OPERATION'
 
     ENGINE_EVENT = 'ENGINE_EVENT'
@@ -96,10 +97,6 @@ def _validate_event_specific_data(event_type, event_specific_data):
         check.inst_param(event_specific_data, 'event_specific_data', StepMaterializationData)
     elif event_type == DagsterEventType.STEP_EXPECTATION_RESULT:
         check.inst_param(event_specific_data, 'event_specific_data', StepExpectationResultData)
-    elif event_type == DagsterEventType.PIPELINE_PROCESS_STARTED:
-        check.inst_param(event_specific_data, 'event_specific_data', PipelineProcessStartedData)
-    elif event_type == DagsterEventType.PIPELINE_PROCESS_EXITED:
-        check.inst_param(event_specific_data, 'event_specific_data', PipelineProcessExitedData)
     elif event_type == DagsterEventType.STEP_INPUT:
         check.inst_param(event_specific_data, 'event_specific_data', StepInputData)
     elif event_type == DagsterEventType.ENGINE_EVENT:
@@ -125,7 +122,7 @@ def log_step_event(step_context, event):
     )
 
 
-def log_pipeline_event(pipeline_context, event):
+def log_pipeline_event(pipeline_context, event, step_key):
     event_type = DagsterEventType(event.event_type_value)
 
     log_fn = (
@@ -139,9 +136,19 @@ def log_pipeline_event(pipeline_context, event):
         ),
         dagster_event=event,
         pipeline_name=pipeline_context.pipeline_def.name,
+        step_key=step_key,
     )
 
 
+def log_resource_event(log_manager, pipeline_name, event):
+    check.inst_param(log_manager, 'log_manager', DagsterLogManager)
+    check.inst_param(event, 'event', DagsterEvent)
+    check.inst(event.event_specific_data, EngineEventData)
+
+    log_fn = log_manager.error if event.event_specific_data.error else log_manager.debug
+    log_fn(event.message, dagster_event=event, pipeline_name=pipeline_name, step_key=event.step_key)
+
+
 @whitelist_for_serdes
 class DagsterEvent(
     namedtuple(
@@ -175,7 +182,9 @@ class DagsterEvent(
         return event
 
     @staticmethod
-    def from_pipeline(event_type, pipeline_context, message=None, event_specific_data=None):
+    def from_pipeline(
+        event_type, pipeline_context, message=None, event_specific_data=None, step_key=None
+    ):
         check.inst_param(pipeline_context, 'pipeline_context', SystemPipelineExecutionContext)
 
         pipeline_name = pipeline_context.pipeline_def.name
@@ -185,12 +194,36 @@ class DagsterEvent(
             check.str_param(pipeline_name, 'pipeline_name'),
             message=check.opt_str_param(message, 'message'),
             event_specific_data=_validate_event_specific_data(event_type, event_specific_data),
+            step_key=step_key,
         )
 
-        log_pipeline_event(pipeline_context, event)
+        log_pipeline_event(pipeline_context, event, step_key)
 
         return event
 
+    @staticmethod
+    def from_resource(execution_plan, log_manager, message=None, event_specific_data=None):
+        from dagster.core.execution.plan.plan import ExecutionPlan
+
+        check.inst_param(execution_plan, 'execution_plan', ExecutionPlan)
+        pipeline_name = execution_plan.pipeline_def.name
+        step_key = (
+            execution_plan.step_keys_to_execute[0]
+            if execution_plan.step_keys_to_execute and len(execution_plan.step_keys_to_execute) == 1
+            else None
+        )
+        event = DagsterEvent(
+            DagsterEventType.ENGINE_EVENT.value,
+            pipeline_name=pipeline_name,
+            message=check.opt_str_param(message, 'message'),
+            event_specific_data=_validate_event_specific_data(
+                DagsterEventType.ENGINE_EVENT, event_specific_data
+            ),
+            step_key=step_key,
+        )
+        log_resource_event(log_manager, pipeline_name, event)
+        return event
+
     def __new__(
         cls,
         event_type_value,
@@ -202,6 +235,10 @@ class DagsterEvent(
         event_specific_data=None,
         message=None,
     ):
+        event_type_value, event_specific_data = _handle_back_compat(
+            event_type_value, event_specific_data
+        )
+
         return super(DagsterEvent, cls).__new__(
             cls,
             check.str_param(event_type_value, 'event_type_value'),
@@ -249,6 +286,22 @@ class DagsterEvent(
     def is_step_failure(self):
         return self.event_type == DagsterEventType.STEP_FAILURE
 
+    @property
+    def is_step_up_for_retry(self):
+        return self.event_type == DagsterEventType.STEP_UP_FOR_RETRY
+
+    @property
+    def is_step_restarted(self):
+        return self.event_type == DagsterEventType.STEP_RESTARTED
+
+    @property
+    def is_pipeline_success(self):
+        return self.event_type == DagsterEventType.PIPELINE_SUCCESS
+
+    @property
+    def is_pipeline_failure(self):
+        return self.event_type == DagsterEventType.PIPELINE_FAILURE
+
     @property
     def is_failure(self):
         return self.event_type in FAILURE_EVENTS
@@ -277,24 +330,8 @@ class DagsterEvent(
         return self.event_specific_data
 
     @property
-    def pipeline_process_started_data(self):
-        _assert_type(
-            'pipeline_process_started', DagsterEventType.PIPELINE_PROCESS_STARTED, self.event_type
-        )
-        return self.event_specific_data
-
-    @property
-    def pipeline_process_exited_data(self):
-        _assert_type(
-            'pipeline_process_exited', DagsterEventType.PIPELINE_PROCESS_EXITED, self.event_type
-        )
-        return self.event_specific_data
-
-    @property
-    def pipeline_process_start_data(self):
-        _assert_type(
-            'pipeline_process_start', DagsterEventType.PIPELINE_PROCESS_START, self.event_type
-        )
+    def step_retry_data(self):
+        _assert_type('step_retry_data', DagsterEventType.STEP_UP_FOR_RETRY, self.event_type)
         return self.event_specific_data
 
     @property
@@ -356,6 +393,20 @@ class DagsterEvent(
             message='Execution of step "{step_key}" failed.'.format(step_key=step_context.step.key),
         )
 
+    @staticmethod
+    def step_retry_event(step_context, step_retry_data):
+        return DagsterEvent.from_step(
+            event_type=DagsterEventType.STEP_UP_FOR_RETRY,
+            step_context=step_context,
+            event_specific_data=step_retry_data,
+            message='Execution of step "{step_key}" failed and has requested a retry{wait_str}.'.format(
+                step_key=step_context.step.key,
+                wait_str=' in {n} seconds'.format(n=step_retry_data.seconds_to_wait)
+                if step_retry_data.seconds_to_wait
+                else '',
+            ),
+        )
+
     @staticmethod
     def step_input_event(step_context, step_input_data):
         return DagsterEvent.from_step(
@@ -387,6 +438,16 @@ class DagsterEvent(
             ),
         )
 
+    @staticmethod
+    def step_restarted_event(step_context, previous_attempts):
+        return DagsterEvent.from_step(
+            event_type=DagsterEventType.STEP_RESTARTED,
+            step_context=step_context,
+            message='Started re-execution (attempt # {n}) of step "{step_key}".'.format(
+                step_key=step_context.step.key, n=previous_attempts + 1
+            ),
+        )
+
     @staticmethod
     def step_success_event(step_context, success):
         return DagsterEvent.from_step(
@@ -477,6 +538,68 @@ class DagsterEvent(
             ),
         )
 
+    @staticmethod
+    def resource_init_start(execution_plan, log_manager, resource_keys):
+        from dagster.core.execution.plan.plan import ExecutionPlan
+
+        return DagsterEvent.from_resource(
+            execution_plan=check.inst_param(execution_plan, 'execution_plan', ExecutionPlan),
+            log_manager=check.inst_param(log_manager, 'log_manager', DagsterLogManager),
+            message='Starting initialization of resources [{}].'.format(', '.join(resource_keys)),
+            event_specific_data=EngineEventData(metadata_entries=[], marker_start='resources'),
+        )
+
+    @staticmethod
+    def resource_init_success(execution_plan, log_manager, resource_instances, resource_init_times):
+        from dagster.core.execution.plan.plan import ExecutionPlan
+
+        metadata_entries = []
+        for resource_key in resource_instances.keys():
+            resource_obj = resource_instances[resource_key]
+            resource_time = resource_init_times[resource_key]
+            metadata_entries.append(
+                EventMetadataEntry.python_artifact(
+                    resource_obj.__class__, resource_key, 'Initialized in {}'.format(resource_time)
+                )
+            )
+
+        return DagsterEvent.from_resource(
+            execution_plan=check.inst_param(execution_plan, 'execution_plan', ExecutionPlan),
+            log_manager=check.inst_param(log_manager, 'log_manager', DagsterLogManager),
+            message='Finished initialization of resources [{}].'.format(
+                ', '.join(resource_init_times.keys())
+            ),
+            event_specific_data=EngineEventData(
+                metadata_entries=metadata_entries, marker_end='resources',
+            ),
+        )
+
+    @staticmethod
+    def resource_init_failure(execution_plan, log_manager, resource_keys, error):
+        from dagster.core.execution.plan.plan import ExecutionPlan
+
+        return DagsterEvent.from_resource(
+            execution_plan=check.inst_param(execution_plan, 'execution_plan', ExecutionPlan),
+            log_manager=check.inst_param(log_manager, 'log_manager', DagsterLogManager),
+            message='Initialization of resources [{}] failed.'.format(', '.join(resource_keys)),
+            event_specific_data=EngineEventData(
+                metadata_entries=[], marker_end='resources', error=error,
+            ),
+        )
+
+    @staticmethod
+    def resource_teardown_failure(execution_plan, log_manager, resource_keys, error):
+        from dagster.core.execution.plan.plan import ExecutionPlan
+
+        return DagsterEvent.from_resource(
+            execution_plan=check.inst_param(execution_plan, 'execution_plan', ExecutionPlan),
+            log_manager=check.inst_param(log_manager, 'log_manager', DagsterLogManager),
+            message='Teardown of resources [{}] failed.'.format(', '.join(resource_keys)),
+            event_specific_data=EngineEventData(
+                metadata_entries=[], marker_start=None, marker_end=None, error=error,
+            ),
+        )
+
     @staticmethod
     def pipeline_init_failure(pipeline_name, failure_data, log_manager):
         check.inst_param(failure_data, 'failure_data', PipelineInitFailureData)
@@ -488,8 +611,8 @@ class DagsterEvent(
             pipeline_name=pipeline_name,
             event_specific_data=failure_data,
             message=(
-                'Pipeline failure during initialization of pipeline "{pipeline_name}. '
-                'This may be due to a failure in initializing a resource or logger".'
+                'Pipeline failure during initialization of pipeline "{pipeline_name}". '
+                'This may be due to a failure in initializing a resource or logger.'
             ).format(pipeline_name=pipeline_name),
         )
         log_manager.error(
@@ -503,12 +626,13 @@ class DagsterEvent(
         return event
 
     @staticmethod
-    def engine_event(pipeline_context, message, event_specific_data=None):
+    def engine_event(pipeline_context, message, event_specific_data=None, step_key=None):
         return DagsterEvent.from_pipeline(
             DagsterEventType.ENGINE_EVENT,
             pipeline_context,
             message,
             event_specific_data=event_specific_data,
+            step_key=step_key,
         )
 
     @staticmethod
@@ -615,21 +739,26 @@ class ObjectStoreOperationResultData(
 
 
 @whitelist_for_serdes
-class EngineEventData(namedtuple('_EngineEventData', 'metadata_entries error')):
+class EngineEventData(
+    namedtuple('_EngineEventData', 'metadata_entries error marker_start marker_end')
+):
     # serdes log
     # * added optional error
+    # * added marker_start / marker_end
     #
-    def __new__(cls, metadata_entries, error=None):
+    def __new__(cls, metadata_entries=None, error=None, marker_start=None, marker_end=None):
         return super(EngineEventData, cls).__new__(
             cls,
-            metadata_entries=check.list_param(
+            metadata_entries=check.opt_list_param(
                 metadata_entries, 'metadata_entries', EventMetadataEntry
             ),
             error=check.opt_inst_param(error, 'error', SerializableErrorInfo),
+            marker_start=check.opt_str_param(marker_start, 'marker_start'),
+            marker_end=check.opt_str_param(marker_end, 'marker_end'),
         )
 
     @staticmethod
-    def in_process(pid, step_keys_to_execute=None):
+    def in_process(pid, step_keys_to_execute=None, marker_end=None):
         check.int_param(pid, 'pid')
         check.opt_list_param(step_keys_to_execute, 'step_keys_to_execute')
         return EngineEventData(
@@ -638,7 +767,8 @@ class EngineEventData(namedtuple('_EngineEventData', 'metadata_entries error')):
                 [EventMetadataEntry.text(str(step_keys_to_execute), 'step_keys')]
                 if step_keys_to_execute
                 else []
-            )
+            ),
+            marker_end=marker_end,
         )
 
     @staticmethod
@@ -669,28 +799,45 @@ class EngineEventData(namedtuple('_EngineEventData', 'metadata_entries error')):
         return EngineEventData(metadata_entries=[], error=error)
 
 
-@whitelist_for_serdes
-class PipelineProcessStartedData(
-    namedtuple('_PipelineProcessStartedData', 'process_id pipeline_name run_id')
-):
-    pass
-
-
-@whitelist_for_serdes
-class PipelineProcessExitedData(
-    namedtuple('_PipelineProcessExitedData', 'process_id pipeline_name run_id')
-):
-    pass
-
-
-@whitelist_for_serdes
-class PipelineProcessStartData(namedtuple('_PipelineProcessStartData', 'pipeline_name run_id')):
-    pass
-
-
 @whitelist_for_serdes
 class PipelineInitFailureData(namedtuple('_PipelineInitFailureData', 'error')):
     def __new__(cls, error):
         return super(PipelineInitFailureData, cls).__new__(
             cls, error=check.inst_param(error, 'error', SerializableErrorInfo)
         )
+
+
+###################################################################################################
+# THE GRAVEYARD
+#
+#            -|-
+#             |
+#        _-'~~~~~`-_
+#      .'           '.
+#      |    R I P    |
+#      |             |
+#      |  Synthetic  |
+#      |   Process   |
+#      |   Events    |
+#      |             |
+###################################################################################################
+
+
+def _handle_back_compat(event_type_value, event_specific_data):
+    if event_type_value == 'PIPELINE_PROCESS_START':
+        return DagsterEventType.ENGINE_EVENT.value, EngineEventData([])
+    elif event_type_value == 'PIPELINE_PROCESS_STARTED':
+        return DagsterEventType.ENGINE_EVENT.value, EngineEventData([])
+    elif event_type_value == 'PIPELINE_PROCESS_EXITED':
+        return DagsterEventType.ENGINE_EVENT.value, EngineEventData([])
+    else:
+        return event_type_value, event_specific_data
+
+
+register_serdes_tuple_fallbacks(
+    {
+        'PipelineProcessStartedData': None,
+        'PipelineProcessExitedData': None,
+        'PipelineProcessStartData': None,
+    }
+)
