@@ -1,4 +1,5 @@
 from dagster import check
+from dagster.config.config_type import ConfigType, ConfigTypeKind
 from dagster.core.decorator_utils import (
     split_function_parameters,
     validate_decorated_fn_positionals,
@@ -6,10 +7,6 @@ from dagster.core.decorator_utils import (
 from dagster.core.errors import DagsterInvalidDefinitionError
 from dagster.utils import ensure_single_item
 
-from .builtin_enum import BuiltinEnum
-from .config import ConfigType, List, Nullable
-from .wrapping import WrappingListType, WrappingNullableType
-
 
 class InputHydrationConfig(object):
     @property
@@ -24,29 +21,24 @@ class InputHydrationConfig(object):
         '''
         return config_value
 
+    def required_resource_keys(self):
+        return frozenset()
 
-def resolve_config_cls_arg(config_cls):
-    if BuiltinEnum.contains(config_cls):
-        return ConfigType.from_builtin_enum(config_cls)
-    elif isinstance(config_cls, WrappingListType):
-        return List(resolve_config_cls_arg(config_cls.inner_type))
-    elif isinstance(config_cls, WrappingNullableType):
-        return Nullable(resolve_config_cls_arg(config_cls.inner_type))
-    else:
-        check.type_param(config_cls, 'config_cls')
-        check.param_invariant(issubclass(config_cls, ConfigType), 'config_cls')
-        return config_cls.inst()
+
+class BareInputSchema(InputHydrationConfig):
+    def __init__(self, config_type):
+        self.config_type = check.inst_param(config_type, 'config_type', ConfigType)
+
+    @property
+    def schema_type(self):
+        return self.config_type
 
 
 def make_bare_input_schema(config_cls):
-    config_type = resolve_config_cls_arg(config_cls)
+    from dagster.config.field import resolve_to_config_type
 
-    class _InputSchema(InputHydrationConfig):
-        @property
-        def schema_type(self):
-            return config_type
-
-    return _InputSchema()
+    config_type = resolve_to_config_type(config_cls)
+    return BareInputSchema(config_type)
 
 
 class OutputMaterializationConfig(object):
@@ -62,20 +54,34 @@ class OutputMaterializationConfig(object):
         '''
         check.not_implemented('Must implement')
 
-
-def _create_input_schema(config_type, func):
-    class _InputSchema(InputHydrationConfig):
-        @property
-        def schema_type(self):
-            return config_type
-
-        def construct_from_config_value(self, context, config_value):
-            return func(context, config_value)
-
-    return _InputSchema()
+    def required_resource_keys(self):
+        return frozenset()
 
 
-def input_hydration_config(config_cls):
+class InputSchemaFromDecorator(InputHydrationConfig):
+    def __init__(self, config_type, func, required_resource_keys):
+        self._config_type = check.inst_param(config_type, 'config_type', ConfigType)
+        self._func = check.callable_param(func, 'func')
+        self._required_resource_keys = check.opt_set_param(
+            required_resource_keys, 'required_resource_keys', of_type=str
+        )
+
+    @property
+    def schema_type(self):
+        return self._config_type
+
+    def construct_from_config_value(self, context, config_value):
+        return self._func(context, config_value)
+
+    def required_resource_keys(self):
+        return frozenset(self._required_resource_keys)
+
+
+def _create_input_schema_for_decorator(config_type, func, required_resource_keys):
+    return InputSchemaFromDecorator(config_type, func, required_resource_keys)
+
+
+def input_hydration_config(config_cls, required_resource_keys=None):
     '''Create an input hydration config that maps config data to a runtime value.
 
     The decorated function should take the execution context and parsed config value and return the
@@ -84,17 +90,19 @@ def input_hydration_config(config_cls):
     Args:
         config_cls (Any): The type of the config data expected by the decorated function. Users
             should provide one of the :ref:`built-in types <builtin>`, or a composite constructed
-            using :py:func:`Selector` or :py:func:`PermissiveDict`.
-    
+            using :py:func:`Selector` or :py:func:`Permissive`.
+
     Examples:
 
     .. code-block:: python
 
-        @input_hydration_config(PermissiveDict())
+        @input_hydration_config(Permissive())
         def _dict_input(_context, value):
             return value
     '''
-    config_type = resolve_config_cls_arg(config_cls)
+    from dagster.config.field import resolve_to_config_type
+
+    config_type = resolve_to_config_type(config_cls)
     EXPECTED_POSITIONALS = ['context', '*']
 
     def wrapper(func):
@@ -108,12 +116,12 @@ def input_hydration_config(config_cls):
                     solid_name=func.__name__, missing_param=missing_positional
                 )
             )
-        return _create_input_schema(config_type, func)
+        return _create_input_schema_for_decorator(config_type, func, required_resource_keys)
 
     return wrapper
 
 
-def input_selector_schema(config_cls):
+def input_selector_schema(config_cls, required_resource_keys=None):
     '''
     A decorator for annotating a function that can take the selected properties
     from a ``config_value`` in to an instance of a custom type.
@@ -121,32 +129,45 @@ def input_selector_schema(config_cls):
     Args:
         config_cls (Selector)
     '''
-    config_type = resolve_config_cls_arg(config_cls)
-    check.param_invariant(config_type.is_selector, 'config_cls')
+    from dagster.config.field import resolve_to_config_type
+
+    config_type = resolve_to_config_type(config_cls)
+    check.param_invariant(config_type.kind == ConfigTypeKind.SELECTOR, 'config_cls')
 
     def _wrap(func):
         def _selector(context, config_value):
             selector_key, selector_value = ensure_single_item(config_value)
             return func(context, selector_key, selector_value)
 
-        return _create_input_schema(config_type, _selector)
+        return _create_input_schema_for_decorator(config_type, _selector, required_resource_keys)
 
     return _wrap
 
 
-def _create_output_schema(config_type, func):
-    class _OutputSchema(OutputMaterializationConfig):
-        @property
-        def schema_type(self):
-            return config_type
+class OutputSchemaForDecorator(OutputMaterializationConfig):
+    def __init__(self, config_type, func, required_resource_keys):
+        self._config_type = check.inst_param(config_type, 'config_type', ConfigType)
+        self._func = check.callable_param(func, 'func')
+        self._required_resource_keys = check.opt_set_param(
+            required_resource_keys, 'required_resource_keys', of_type=str
+        )
 
-        def materialize_runtime_value(self, context, config_value, runtime_value):
-            return func(context, config_value, runtime_value)
+    @property
+    def schema_type(self):
+        return self._config_type
 
-    return _OutputSchema()
+    def materialize_runtime_value(self, context, config_value, runtime_value):
+        return self._func(context, config_value, runtime_value)
+
+    def required_resource_keys(self):
+        return frozenset(self._required_resource_keys)
 
 
-def output_materialization_config(config_cls):
+def _create_output_schema(config_type, func, required_resource_keys):
+    return OutputSchemaForDecorator(config_type, func, required_resource_keys)
+
+
+def output_materialization_config(config_cls, required_resource_keys=None):
     '''Create an output materialization hydration config that configurably materializes a runtime
     value.
 
@@ -157,14 +178,14 @@ def output_materialization_config(config_cls):
     Args:
         config_cls (Any): The type of the config data expected by the decorated function. Users
             should provide one of the :ref:`built-in types <builtin>`, or a composite constructed
-            using :py:func:`Selector` or :py:func:`PermissiveDict`.
+            using :py:func:`Selector` or :py:func:`Permissive`.
 
     Examples:
 
     .. code-block:: python
 
         # Takes a list of dicts such as might be read in using csv.DictReader, as well as a config
-        value, and writes 
+        value, and writes
         @output_materialization_config(Path)
         def df_output_schema(_context, path, value):
             with open(path, 'w') as fd:
@@ -175,11 +196,13 @@ def output_materialization_config(config_cls):
             return Materialization.file(path)
 
     '''
-    config_type = resolve_config_cls_arg(config_cls)
-    return lambda func: _create_output_schema(config_type, func)
+    from dagster.config.field import resolve_to_config_type
+
+    config_type = resolve_to_config_type(config_cls)
+    return lambda func: _create_output_schema(config_type, func, required_resource_keys)
 
 
-def output_selector_schema(config_cls):
+def output_selector_schema(config_cls, required_resource_keys=None):
     '''
     A decorator for a annotating a function that can take the selected properties
     of a ``config_value`` and an instance of a custom type and materialize it.
@@ -187,14 +210,16 @@ def output_selector_schema(config_cls):
     Args:
         config_cls (Selector):
     '''
-    config_type = resolve_config_cls_arg(config_cls)
-    check.param_invariant(config_type.is_selector, 'config_cls')
+    from dagster.config.field import resolve_to_config_type
+
+    config_type = resolve_to_config_type(config_cls)
+    check.param_invariant(config_type.kind == ConfigTypeKind.SELECTOR, 'config_cls')
 
     def _wrap(func):
         def _selector(context, config_value, runtime_value):
             selector_key, selector_value = ensure_single_item(config_value)
             return func(context, selector_key, selector_value, runtime_value)
 
-        return _create_output_schema(config_type, _selector)
+        return _create_output_schema(config_type, _selector, required_resource_keys)
 
     return _wrap
